!> @brief surface flux model common subroutines
module sfx_common
    public

contains

    !> @brief string to int conversion
    elemental subroutine str2int(int, str, stat)
        ! ----------------------------------------------------------------------------
        implicit none
        integer, intent(out) :: int
        integer, intent(out) :: stat                !> output status, /= 0 signals ERROR

        character(len = *), intent(in) :: str
        ! ----------------------------------------------------------------------------

        read(str, * , iostat = stat) int
    end subroutine str2int
    ! ----------------------------------------------------------------------------

    !> @brief string to real conversion
    elemental subroutine str2real(x, str, stat)
        ! ----------------------------------------------------------------------------
        implicit none
        real, intent(out) :: x
        integer, intent(out) :: stat                !> output status, /= 0 signals ERROR

        character(len = *), intent(in) :: str
        ! ----------------------------------------------------------------------------

        read(str, * , iostat = stat) x
    end subroutine str2real
    ! ----------------------------------------------------------------------------

    !> @brief character array to string conversion
    function char_array2str(char_array) result(str)
        ! ----------------------------------------------------------------------------
        implicit none
        character, intent(in) :: char_array(:)
        character(len=:), allocatable :: str
        integer :: i
        ! ----------------------------------------------------------------------------

        str = ""
        do i = 1, size(char_array)
            str = str(:) // char_array(i)
        end do

    end function
    ! ----------------------------------------------------------------------------


    !> @brief check if value is finite
    elemental function is_finite(value)
        ! ----------------------------------------------------------------------------
        use ieee_arithmetic
        implicit none
        logical :: is_finite

        real, intent(in) :: value
        ! ----------------------------------------------------------------------------

        is_finite = ieee_is_finite(value)
    end function is_finite
    ! ----------------------------------------------------------------------------

end module sfx_common